/*  smplogx.cpp  by K.Tsuru  */
const char* title =
"<<<< A sample program for Log(x), Log10() and DpowD(x, p) functions. >>>>\n";

#include "sn.h"

void ShowCalcInfo(const char* const result, const SDouble& c, double cpu_time){
	cout << "Above is " << result << " up to " << c.UpToTerm();
	cout << "th terms, cpu time = " << cpu_time << endl;
}
void Exp1ByDpowD(){
	SDouble X;
	cout << "Compare e with (1+1/x)^x." << endl;

	while(1){
		cerr << "Input a large integral value(e.g \"3.5e1000\") x = ? ";
		X = GetString();
		if(X.Sign() <= 0){
			ErrorPuts("x must be positive!");
			continue;
		} else break;
	}

	RealSize C;
	uint fig = (uint)(2*X.RdxExp()) + C.EffFigures();
	C.SetEffFig(fig);

	SDouble x(X), y, z, e = E();
	y = 1.0 + 1.0/x;	// y = 1 + 1/x
	cout << "x= " << x << endl;
	cout << "1+1/x =" << y << endl;
	z = DpowD(y, x);	// z = (1 + 1/x)^x
	cout << "e=" << e << endl;
	cout << " (1+1/x)^x = " << z << endl;
	cout << AgreeUpTo(e, z) << " figures agree. Effective figures = " << z.DFigures() << endl;
	Wait();
	C.SetEffFig(0);
}
void logXSpeedTest(){
	RealSize C;
	Timer timer;
	double cpu_time;
	uint fig;
	cerr << "A test for Log(x). Effective figures /" << DFIGURES << " = ? ";
	fig = GetInt();
	C.SetEffFig(fig);
	SDouble x, ln;

	cerr << "Preparing E() in your effective figures." << endl;
	E();
	while(1){
		cerr << "Input x = (Abort when x <= 0)? ";
		cin >> x;
		if(x <= 0.0) break;
		cout << "Your x = " << x << endl;
		timer.Start();
		ln = Log(x);
		cpu_time = timer.Stop();
		cout << ln << endl;
		ShowCalcInfo("Log(x)", ln, cpu_time);
		Wait();
	}
	C.SetEffFig(0);
}
void LogExpTest(){
	Timer timer;
	RealSize C;
	uint fig;

	cerr << "A test program for Log(x) and Exp(x)." << endl;
	cerr << "In the following results hidden figures are shown." << endl;
	cerr << "Effective figures /" << DFIGURES << " = ? ";
	fig	= abs(GetInt());
	C.SetEffFig(fig);

	SDouble  x, y, z, ln, ln2, d, e;
	double cpu_time;

	cout << "********************* Calculating E() ***************************" << endl;
	timer.Start();
	e = E();
	cpu_time = timer.Stop();
	cout << e << endl;
	ShowCalcInfo("E()", e, cpu_time);
	Wait();

	cout << "******************** Calculating Log(e) ************************" << endl;
	timer.Start();
	ln = Log(e);
	cpu_time = timer.Stop();
	cout << ln << endl;
	ShowCalcInfo("Log(e)", ln, cpu_time);
	cout << "Did you get a value very close to 1.0?" << endl;
	Wait();

	cout << "******************* Calculating Log(2.0) ************************" << endl;
	timer.Start();
	ln2 = Log(2.0);
	cpu_time = timer.Stop();
	cout << ln2 << endl;
	ShowCalcInfo("Log(2.0)", ln2, cpu_time);
	Wait();

	cout << "****************** Calculating Exp(Log(2.0)) *******************" << endl;
	timer.Start();
	z = Exp(ln2);
	cpu_time = timer.Stop();
	cout << z << endl;
	ShowCalcInfo("Exp(Log(2.0))", ln, cpu_time);
	cout<< "Did you get a value very close to 2.0?" << endl;
	Wait();

	cout << "******************* Calculating 2^(1/3) *************************" << endl;
	cout << "In first step : y = Log(2.0)/3." << endl;
	y = ln2/3;
	cout << y << endl;
	Wait();

	cout << "In second step : x = Exp(y)." << endl;
	x = Exp(y);
	cout << "****************** Result of x = 2^(1/3) ************************" << endl;
	cout << x << endl;
	Wait();

	cout << "******************* Calculating y = x^3 *************************" << endl;
	y = Dpow(x,3);
	cout << "***************** Result of x^3 (x = 2^(1/3) ********************" << endl;
	cout << y << endl;
	cout << "Did you get a value very close to 2.0?" << endl;
	Wait();

	cout << "************ Calculating 2^(1/3) by DpowD(x, p) *****************" << endl;
	timer.Start();
	z = DsDiv(SDouble(1.0), 3);		// z = 1/3 It needs a cast double to SDouble. Since version 2.20.
	y = DpowD(2.0, z);
	cpu_time = timer.Stop();
	cout << "cpu time = " << cpu_time << endl;
	d = x - y;
	cout << "Difference = " << d << endl;
	cout << "Did you get a very small value?" << endl;
	Wait();

	cout << "***************** Calculating Exp(-2.5) *************************" << endl;
	y = Exp(-2.5);
	cout << "****************** Result of Exp(-2.5) **************************" << endl;
	cout << y << endl;
	Wait();

	cout << "*************** Calculating Log(Exp(-2.5)) *********************" << endl;
	ln = Log(y);
	cout << "**************** Result of Log(Exp(-2.5)) **********************" << endl;
	cout << ln << endl;
	cout << "Did you get a value very close to -2.5?" << endl;
	Wait();
	C.SetEffFig(0);
}

int main(){
	SDouble F;
	F.SetFormat(5, 0, 10, F.CRLF|F.ROUND|F.SHOW_FIG);

	cerr << title << endl;
	Exp1ByDpowD();
	logXSpeedTest();
	LogExpTest();
	Ending();
}
