/*  smpbern.cpp  by K.Tsuru  */
const char* title = "<<< A sample program for tangent coefficient, Bernouilli and Euler numbers >>>\n";
#include "sn.h"

void TnTableTest(uint N){
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;

	cout << "Calculating tangent coefficients by TanCoeffTable() function." << endl;
	timer.Start();
	TanCoeffTable(T, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++){
		cout << "T[" << n << "]= " << T(n) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void TanCoeffTest(uint N){
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calcuating tangent coefficients by TanCoeff() function." << endl;
	timer.Start();
	for(n = 1; n <= N; n++) T[n-1] = TanCoeff(n);
	cpu_time = timer.Stop();

	for(n = 1; n <= N; n++){
		cout << "T[" << n << "]= " << T(n-1) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void BnTableTest(uint N){
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;

	cout << "Calculating Bernouilli numbers by BernNumTable() function." << endl;
	timer.Start();
	BernNumTable(B, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++){
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void BernNumTest(uint N){
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;
	uint n;
	puts("Calculating Bernouilli numbers by BernNum() function with combL().");
	timer.Start();
	for(n = 1; n <= N; n++) B[n] = BernNum(n);
	cpu_time = timer.Stop();
	for(n = 1; n <= N; n++){
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void EulerNumTest(uint N){
	SNBlock <SLong> E(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	puts("Calculating Euler numbers by EulerNum() function.");
	timer.Start();
	for(n=0; n<= N; n++) E[n] = EulerNum(n);
	cpu_time = timer.Stop();

	for(n = 0; n<= N; n++){
		cout << "E[" << n << "]= " << E(n) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}
/*

*/
SLong binom(ulong n, ulong k) {
	k = min(n, n-k);
	SLong r;
	if(k == 0) r = 1;
	else {
	 	r = binom(n-1, k-1);
	 	r *=n;	r/= k;
	}
	return r;
}

void CombTest(ulong n, ulong k) {
	SLong cmb = comb(n, k);
	cout << binom(n,k) << endl;
	cout << n << "_C_" << k << "=" << cmb << endl;
}
int main(){
	uint N;
	cout.precision( 3 );

	ErrorPuts(title);
	cerr << "How many terms do you want? ";
	N = GetInt();

	BnTableTest(N);
	BernNumTest(N);
	TnTableTest(N);
	TanCoeffTest(N);
	EulerNumTest(N);

	Ending();
}
