/*  ChudPiSpeedTest.cpp  by K.Tsuru  */
#include "sn.h"
#include "wait.h"
#include "timer.h"
/*
Speed test of pi by Chudnovskys' formula.
32bits version
132,000,000 digits ... out of memory... FFT_MAX_BITS = 23
100,000,000... out of memory... FFT_MAX_BITS = 22

 33,550,000 digits 156.786(sec) 2.6min... FFT_MAX_BITS = 23
				   239.843(sec) 4.0min... FFT_MAX_BITS = 22
 50,000,000        365.903      6.1min
2017/03/22 version=========================
 50,000,000        301.794      5.03min
 10,000,000         52.02       0.867min
2017/04/02 version=========================
 10,000,000         57.09(sec)  [0.95143 min]10000029(figures)
 50,000,000        324.443(sec) [5.40739 min]50000029(figures)
KaratsubaMult() is not used.
 10,000,000         58.8582(sec) [0.98097 min]10000029(figures) slightly slow
2017/04/06 version=========================
Now computing in 50000000 digits start at 16:05:26 ... added time
ChudnovskysPi CPU time = 320.764(sec) [5.34607 min]50000029(figures)
2017/06/19 version====MinGW64=====================
Now computing in 50000000 digits   Start at 11:21:46
ChudnovskysPi CPU time = 454.423(sec) [7.57371 min]50000029(figures)
Now computing in 100000000 digits   Start at 15:54:29
ChudnovskysPi CPU time = 1139(sec) [18.9833 min]100000029(figures)
2017/06/19 version====MinGW32=====================
Now computing in 50000000 digits   Start at 11:40:29
ChudnovskysPi CPU time = 485.833(sec) [8.09722 min]50000029(figures)
2017/07/07 version====MinGW64 + long double FFT=====================
Now computing in 50000000 digits   Start at 11:49:57
ChudnovskysPi CPU time = 765.708(sec) [12.7618 min]50000029(figures) 1.576 Times
LDBL_MANT_DIG = 64

Now computing in 1000000000 digits   Start at 16:16:57
ChudnovskysPi CPU time = 34351(sec) [572.517 min]1000000029(figures)
2017/06/28 version=====TDM-GCC 64bit===============
Now computing in 50000000 digits   Start at 10:44:27
ChudnovskysPi CPU time = 445.417(sec) [7.42361 min]50000029(figures)

2017/07/07 version====MinGW64 + long double FFT + SinTable=====================
Now computing in 50000000 digits   Start at 16:33:43
maxsizebit = 27  L= 27 tableSize =33554433
ChudnovskysPi CPU time = 764.574(sec) [12.7429 min]50000029(figures)
LDBL_MANT_DIG = 64

2017/07/30 version====TDM-GCC 64bis + long double FFT + SinTable=====================
Now computing in 50000000 digits   Start at 10:57:53
maxsizebit = 27  L= 27 tableSize =33554433
ChudnovskysPi CPU time = 832.715(sec) [13.8786 min]50000029(figures)

2017/08/05 version====MinGW64 + SinTable=====================
Now computing in 50000000 digits   Start at 15:15:59
ChudnovskysPi CPU time = 418.88(sec) [6.98134 min]50000029(figures)
<<< FFT information >>>
defaultFFTMinSize = 16
fftMaxSize = 8388608(DRADIX) and 1048576(BRADIX)
MFFTMinSize() = 16,  FFTUsedTimes() = 7940246
FFTVerify is OFF.
FFTWorkSize is 25165846.
FFT used memory is 201326768(bytes).
Uses Karatsuba's multiplication = Yes
Karatsuba's multiplication used times = 93.

2017/08/09 version====MinGW64 + SinTable=====================
Now computing in 100000000 digits   Start at 15:1:37
ChudnovskysPi CPU time = 1006(sec) [16.7667 min]100000029(figures)
<<< FFT information >>>
defaultFFTMinSize = 16
fftMaxSize = 8388608(DRADIX) and 1048576(BRADIX)
MFFTMinSize() = 16,  FFTUsedTimes() = 16204977
FFTVerify is OFF.
FFTWorkSize is 25165846.
FFT used memory is 201326768(bytes).
Uses FFTSineTable : Yes
Uses Karatsuba's multiplication : Yes
Karatsuba's multiplication used times = 237.

2017/08/09 version====MinGW64 + SinTable+long double=====================
Now computing in 100000000 digits   Start at 15:50:14
ChudnovskysPi CPU time = 1729(sec) [28.8167 min]100000029(figures)
<<< FFT information >>>
defaultFFTMinSize = 16
fftMaxSize = 134217728(DRADIX) and 16777216(BRADIX)
MFFTMinSize() = 16,  FFTUsedTimes() = 16203574
FFTVerify is OFF.
FFTWorkSize is 100663322.
FFT used memory is 1610613152(bytes).
Uses FFTSineTable : Yes
Uses Karatsuba's multiplication : Yes
Karatsuba's multiplication used times = 0.
*****/

void ShowTitle(const char* title){
	cout << "****************** " << title << " ********************" << endl;
}
void FileOut(const SDouble& data, const string& fname, const string& title) {
	ofstream fout(fname.c_str(),ios::trunc);	 // open file

	data.SetFormat(10, 0, 10, data.INT_PUT|data.CRLF);
//	data.SetFormat(10, 0, 5, data.INT_PUT|data.CRLF|data.SHOW_FIG); // add the number of figures at line end
	fout << title << " (" << data.DFigures() << " digits)" << endl;
	fout << data << endl << " (" << data.DFigures() << " digits)" << endl;
	fout.close();
/*
cout << "line number = " << data.CRCount(false) << endl;
	ifstream fin(fname.c_str(),ios::in);
//	string foname;
	char buff[201], splitFname[201];
//	int line, end = data.CRCount(false);
	int j = 1;
	sprintf(splitFname, "ChudPi%02d.dat", j);

	fout.open(splitFname);
	while(!fin.eof()) {
		fin.getline(buff, 200, '\n');
//		cout << buff << endl;
		fout << buff << endl;
	}
*/
}
SDouble ConstFuncSpeed(SDouble (*sdfunc)(), const char* fname) {
	Timer timer;
	SDouble r;
	ShowTitle(fname);
	cerr << "Now computing in " << 4*r.EffFig() << " digits ";
/*	time_t st = timer.Start();
	struct tm *pnow = localtime(&st);
	printf("start at %02d:%02d:%02d \n",pnow->tm_hour, pnow->tm_min, pnow->tm_sec);
*/
	timer.Start();
	cout << "  Start at " << WhatTimeNow() << endl;
	r = (*sdfunc)();
	double time = timer.Stop();

	cout << fname << " CPU time = " << time << "(sec) [" << (time/60.0) << " min]" << r.DFigures() << "(figures)"<< endl;

	return r;
}
typedef long long llint;
#if defined _WIN64
#define OS 64
#else
#define OS 32
#endif
#if 1
void LongDoubleTest() {
	cout << "*********** LongDoubleTest() *************" << endl;
	llint a = (llint)1 << 63; // on 32 bit OK
	cout << a << endl;
	cout << "LDBL_DIG = " << LDBL_DIG << endl;
	double d, f, td, tld;
	Timer timer;
	long long int i, N=100000000;

	srand((unsigned)time(NULL));
	timer.Start();
	for(i =0; i<N; i++) { 
		d = (double)rand()/RAND_MAX;
		f = sin(d);
	}
	cout << "sin( " << d << " ) = " << f << endl;		
	td = timer.Stop();
	long double ld, lf;
	timer.Start();
	for(i =0; i<N; i++) { 
		ld = (double)rand()/RAND_MAX;
		lf = sinl(ld);
	}	
	cout << "sinl( " << ld << " ) = " << lf << endl;		
	tld = timer.Stop();
	cout << "cpu time : double " << td << "  long double " << tld << " ratio ld/d = "<< tld/td << endl;
}
#endif
int main() {
	cout << "Your system is " << OS << " bit." << endl;
//LongDoubleTest();
	RealSize C;
	SLong t;
	t.KaratsubaMultSwitch(ON);
	string msg = "KaratsubaMult() is ";
	msg += t.UsesKaratsubaMult() ? "ON" : "OFF";
	cout << msg << endl;
//cout << "split files = " << SplitLargeFile("ChudnovskysPi_2KR_ON.dat", 1000000,"ChudPi%02d.dat") << endl;
	cerr << "How many digits do you want? = ";
	int fig = GetInt();
	if(fig <= 0) return -1;
	C.SetEffFig(fig/4);
//	SDouble a = DReciprocal(Sqrt(3.5)), b = Sqrt(3.5);
//cout << a*b << endl;
//Wait();	
	ShowTitle("Speed test of pi by Chudnovskys' formula.");
	SDouble piCh = ConstFuncSpeed(ChudnovskysPi, "ChudnovskysPi");
//	SDouble pi = ConstFuncSpeed(M2Pi, " 2/pi");
//	cout << pi << endl;
FFTInfo();
cout << t.SNObjects() << endl;
	Wait("Do you output to file ?");
	FileOut(piCh, "ChudnovskysPi.dat", "Pi =");
	Ending();
}
