/*  DpowTest.cpp  by K.Tsuru  */ 

#include "sn.h"

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
bool isSmall(const SDouble& x) {
	static const SDouble err = 1.0e-11;
	return ( Dabs(x) < err );
}

void Dpow_nTest() {
	const char* fname = "Dpow(const SDouble& x, long n)";
	Timer timer;
	ShowTitle(fname);
	SDouble x, y, diff;
	long m, n, N = 4;
	double a = M_PI, xd, yd;

	cout << "  x        n       x^n value             relative error" << endl;
	x.SetFormat(8,8);
	cout.precision(7);	
	timer.Start();

	for(m = -N; m <= N; m++) {
		for(n=-N; n<= N; n++) {
			if(m==0 || n ==0) continue;
			xd = a * m;
			x = xd;
			y = Dpown(x, n);
			yd = pow(xd, n);
			diff = y/yd - 1.0;
			cout << xd <<"  "<< n << "  " << yd << "  "<< diff << endl;
			if(!isSmall(diff)){
				cerr << "error is large" << endl; Wait();
			}
		}
	}
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Dpow_pTest() {
	const char* fname = "Dpow(const SDouble& x,const SDouble& p)";
	Timer timer;
	ShowTitle(fname);
	SDouble x, p, y, diff;
	long m, n, N = 4;
	double a = M_PI, b = M_SQRT2, xd, pd, yd;

	cout << "  x        p       x^p value             relative error" << endl;
	x.SetFormat(8,8);
	cout.precision(7);	
	timer.Start();

	for(m = 1; m <= N; m++) {
		xd = a * m;
		x = xd;
		for(n=-N; n<= N; n++) {
			if(m==0 || n ==0) continue;
			pd = n*b;
			p = pd;
			y = DpowD(x, p);
			yd = pow(xd, pd);
			diff = y/yd - 1.0;
			cout << xd <<"  "<< pd << "  " << yd << "  "<< diff << endl;
			if(!isSmall(diff)){
				cerr << "error is large" << endl; Wait();
			}
		}
	}
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Dpow10_Test() {
	const char* fname = "SDouble Dpow10(const SDouble& x);";
	Timer timer;
	ShowTitle(fname);
	SDouble x, y, diff;
	double xd, yd;
	int i, N = 10;
	double a = sqrt(2.0);

	cout << "  x          10^x value    relative error" << endl;
	x.SetFormat(8,8);
	cout.precision(7);	
	timer.Start();

	for(i = -N; i <= N; i++) {
		if(i==0) continue;
		xd = a * i;
		x = xd;
		y = Dpow10(x);
		yd = pow(10.0, xd);
		diff = y/yd - 1.0;
		cout << xd <<"  "<< y << "  " << diff << endl;
		if(!isSmall(diff)) {
			cerr << "error is large" << endl; Wait();
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
int main() {
	Dpow10_Test();	Wait();
	Dpow_nTest();	Wait();
	Dpow_pTest();
}
