/*  ExpRecETest.cpp  by K.Tsuru  */
#include "sn.h"

/*
Test program the base of natural logarithm e.
  E()*RecE()=1 or not.
This is also the test of maximum digits.
  
const int FFT_MAX_SIZE_BITS = 23;	// 20 --> 22 --> 23;
CPU time
100,000,000 digits
E()    296.0(sec) about 4.9(min)
RecE() 295.6(sec) nearly the same
1 - E()*RecE() = -1.0e-100000028 OK

============================================
130,000,000 digits(FFT_MAX_SIZE_BITS = 22)
E()    444.42(sec) about 7.4(min)
RecE() 439.81(sec) nearly the same
1 - E()*RecE() =  41.0 (sec)
 -1.0e-130000028 OK
=============================================
130,000,000 digits(FFT_MAX_SIZE_BITS = 23)
E()    342.483(sec) about 5.7(min)... 1.3 times faster
RecE() 342.811(sec) nearly the same
1 - E()*RecE() =  25.6 (sec)
 -1.0e-130000028 OK

=============================================
132,000,000 digits normally ended.
copy of screen of execution
------------------------------------
 The test of RecE()*E()=1 ? This is a test of the limit of figures.
Where RecE() and E() are independently evaluated by series using "class BinarySplitting".
How many digits do you want? (<=0 quit) = 132,000,000
Computing e now in 132000000(digits).
Cpu time of E()=285.825(sec)
Computing 1/e now.
Cpu time of RecE()=285.461(sec)
Computing 1 - e*(1/e) now.
Cpu time of 1.0 - e*(1/e))= 25.7535(sec)
0.0 <== close to 0.0 ?
=============================================
132,000,000 digits(FFT_MAX_SIZE_BITS = 22) again 2017/03/19
 The test of RecE()*E()=1 ? This is a test of the limit of figures.
Where RecE() and E() are independently evaluated by series using "class BinarySplitting".
How many digits do you want? (<=0 quit) = 132,000,000
Computing e now in 132000000(digits).
Cpu time of E()=459.523(sec)
Computing 1/e now.
Cpu time of RecE()=458.32(sec)
Computing 1 - e*(1/e) now.
Cpu time of 1.0 - e*(1/e))= 41.8729(sec)
0.0 <== close to 0.0 ?
=============================================
132,000,000 digits(FFT_MAX_SIZE_BITS = 23) again 2017/03/19
 The test of RecE()*E()=1 ? This is a test of the limit of figures.
Where RecE() and E() are independently evaluated by series using "class BinarySplitting".
How many digits do you want? (<=0 quit) = 132,000,000
Computing e now in 132000000(digits).
Cpu time of E()=355.502(sec)
Computing 1/e now.
Cpu time of RecE()=354.356(sec)
Computing 1 - e*(1/e) now.
Cpu time of 1.0 - e*(1/e))= 25.7738(sec)
0.0 <== close to 0.0 ?
=============================================
2017/09/21 1,000,000,000 digits(64 bit system)
 The test of RecE()*E()=1 ? This is a test of the limit of figures.
Where RecE() and E() are independently evaluated by series using "class BinarySplitting".
How many digits do you want? (<=0 quit) = 100000 0000
Computing e now in 1000000000(digits).
Cpu time of E()=4936(sec)
Computing 1/e now.
Cpu time of RecE()=4959(sec)
Computing 1 - e*(1/e) now.
Cpu time of 1.0 - e*(1/e)= 701.03(sec)
1.0e-1000000028 <== close to 0.0 ?

*/
void Exp_Mult_RecE_eq1(SDouble &e, SDouble &recE, int* f)
{
	cerr << " The test of RecE()*E()=1 ? This is a test of the limit of figures." << endl;
	cerr << "Where RecE() and E() are independently evaluated by series using \"class BinarySplitting\"." << endl;
	RealSize C;
	Timer timer;
	cerr << "How many digits do you want? (<=0 quit) = ";
	int fig = GetInt();
	if(!fig) return;
	C.SetEffFig(fig/4);
	*f =fig/4;
	SDouble x;
// e = 1 + 1/1! + 1/2! + 1/3! +...
	cerr << "Computing e now in " << fig << "(digits)." << endl;
	timer.Start();
	e = BSE();	// default method of e
	double te = timer.Stop();
	cout << "Cpu time of E()=" << te <<"(sec) " << endl;
// 1/e = 1 - 1/1! + 1/2! - 1/3! +...
	cerr << "Computing 1/e now." << endl;
	timer.Start();
	recE = RecE(false);  // It evaluates by the series not 1/E(). See "snmath.h"
	double tre = timer.Stop();
	cout <<  "Cpu time of RecE()=" << tre <<"(sec)" << endl;

	cerr << "Computing 1 - e*(1/e) now." << endl;
	timer.Start();
	x=1.0 - e*recE; // e*(1/e)
	double chk = timer.Stop();
	cout <<  "Cpu time of 1.0 - e*(1/e)= " << chk <<"(sec)" << endl;

	cout << x << " <== close to 0.0 ?" << endl;
}

void FileOut(const SDouble& data, const string& fname, const string& title) {
	ofstream fout(fname.c_str(),ios::trunc);	 // open file

	data.SetFormat(10, 0, 5, data.INT_PUT|data.CRLF);
//	data.SetFormat(10, 0, 5, data.INT_PUT|data.CRLF|data.SHOW_FIG); // add the number of figures at line end
	fout << title << " (" << data.DFigures() << " digits)" << endl;
	fout << data << endl << " (" << data.DFigures() << " digits)" << endl;
	fout.close();
}
int main() {
	SDouble e, recE;
	int f;
	Exp_Mult_RecE_eq1(e, recE, &f);
	SNInfo();

	const string e_fname ="e.dat", recE_fname ="recE.dat";
	cerr << "Would you like output the \"e\" result to a file \"" << e_fname << "\" Yes(1)/No(0) ?";
	if( !GetInt() ) return 0;

	RealSize C;
	C.SetEffFig(f);
	FileOut(e, e_fname, "e =");

	cerr << "Output \"1/e\" too a file \"" << recE_fname << "\" Yes(1)/No(0) ?";
	if( !GetInt() ) return 0;
	FileOut(recE, recE_fname, "1/e =");
}
