/* FFTSpeedTest.CPP by K.Tsuru*/
// Test on the speed of FFT multiplication.

#include "sn.h"
const int N = 100000;
/**************************************************
 "fig_ratio" is to test the effect of a statement
     if( divN > 1) return DivLLMult(m, n);
 in function LLMult() (slfllmul.cpp).
***************************************************/
double randLLMultFig(int fig, uint* fs, int times, uint fftMin, int sq, int fig_ratio);
double randIIMultFig(int fig, uint* fs, int times, uint fftMin, int sq,int fig_ratio);
void randLLMultSpeedTest(fType rdx){
	printf("Test on figure dependence %s multiplication speed.\n", (rdx == DRADIX) ? "SLong" : "SInteger");
// To know which is faster notmal or FFT, set "fig0 = 8, step =8, fig_ratio=1".
// To know the effect DivLLMult() in LLMult()(slfllmul.cpp)
// set "fig = 128, step=128, fig_ratio =2".
#if 1
	const int fig0 = 128, fig_ratio=2, step=128, fig_max=1024;
#else
	const int fig0 = 8, fig_ratio=1, step=8, fig_max=128;
#endif
        int fig = fig0, times;
	double t, nofft;
	SLong C;
	uint fs, s;

	puts("  figures   CPU time(not use FFT)  times  FFTSize()  CPU ratio");
	while(fig <= fig_max){
		times = N/fig;
		if(rdx == DRADIX){
			t = randLLMultFig(fig, &fs, times, 2, 0, fig_ratio);
			nofft = randLLMultFig(fig, &s, times, 0, 0, fig_ratio);
		} else {
			t = randIIMultFig(fig, &fs, times, 2, 0, fig_ratio);
			nofft = randIIMultFig(fig, &s, times, 0, 0, fig_ratio);
		}
		printf("%4d*%4d %8.4f(%8.4f) %10d %10u", fig, fig/fig_ratio, t, nofft, times , fs);
		if(t > 5.0e-4) printf("%10.3f\n", nofft/t);
		else           puts("   -------");
		fig += step;
	}
	C.SetFFTMinSize(C.MFFTMinSize());
	Wait();
}
void randLLMultSpeed(){
	puts("Test on figure dependence SLong multiplication speed.");
	SLong C;
	int fig, times;
	double t, tsq;
	uint fs;
	while(1){
		printf("figures / 4 =(When 0 abort)? ");
		fig = GetInt();
		if(fig == 0) break;
		times =  N/fig;
		t = randLLMultFig(fig, &fs, times, C.FFTMinSize(), 0, 1);
		tsq = randLLMultFig(fig, &fs, times, C.FFTMinSize(), 1, 1);
		printf("%d times = %g\n", times, t);
		printf("%d times (case x*x) = %g\n", times, tsq);
		printf("FFTSize() = %u\n", fs);
		printf("FFTMinSize() = %u\n", C.FFTMinSize());
	}
}
void randLLMultCompare(){
	puts("Testing that normal and FFT multiplication results agree each other on SLong.");
	int fig;
	long fu;
	SLong a, b, f, n;
	uint  fftfig = SNManager::MFFTMinSize();
	while(1){
		printf("figures / 4 =(When 0 abort)? ");
		fig = GetInt();
		if(fig == 0) break;
		a.FFTUsedTimes(0);
		a.SetFFTMinSize(fftfig);
		a = LRand(fig);
		b = LRand(fig);
		f = a*b;
//f.RawPuts();
		fu = a.FFTUsedTimes();
		printf("FFTMinSize() = %u\n", a.FFTMinSize());
		printf("FFTUsedTimes()= %ld\n", fu);

		a.FFTUsedTimes(0);
		a.SetFFTMinSize(0);
		n = a*b;
//n.RawPuts();
		fu = a.FFTUsedTimes();
		printf("FFTMinSize() = %u\n", a.FFTMinSize());
		printf("FFTUsedTimes()= %ld\n", fu);
		n -= f;
		n.Put();	puts(" = 0 ?");
	}
	a.SetFFTMinSize(fftfig);
	Wait();
}
double randLLMultFig(int fig, uint* fs, int times, uint fftMin, int sq,int fig_ratio){
	SLong a, b, q;
	int i = times;
	double t;
	Timer timer;

	if(i==0) i=1;
	a.SetFFTMinSize(fftMin);
	a.FFTUsedTimes(0);
	a = LRand(fig); 	b = LRand(fig/fig_ratio);
//a.RawPuts();
	if(sq){
		timer.Start();
		while(i){	q = a*a;	i--;	}
		t = timer.Stop();
	}else{
		timer.Start();
		while(i){	q = a*b;	i--;	}
		t = timer.Stop();
	}
	*fs = a.FFTSize();
	return t;
}
double randIIMultFig(int fig, uint* fs, int times, uint fftMin, int sq,int fig_ratio){
	SInteger a, b, q;
	int i=times;
	double t;
	Timer timer;

	if(i==0) i=1;
	a.SetFFTMinSize(fftMin);
	a.FFTUsedTimes(0);
	a = LRand(fig, BRADIX); 	b = LRand(fig/fig_ratio, BRADIX);
	if(sq){
		timer.Start();
		while(i){	q = a*a;	i--;	}
		t = timer.Stop();
	}else{
		timer.Start();
		while(i){	q = a*b;	i--;	}
		t = timer.Stop();
	}
	*fs = a.FFTSize();
	return t;
}
int main(){
	SNManager C;
	int fftFig;

	randLLMultSpeedTest(DRADIX);
	randLLMultSpeedTest(BRADIX);
	randLLMultCompare();
	printf("FFTMinSize() = ? ");
	fftFig = GetInt();
	C.SetFFTMinSize(fftFig);
	printf("FFTMinSize() = %u\n", C.MFFTMinSize());
	randLLMultSpeed();
	Ending();
	return 0;
}
