/*  smpcmpx2.cpp  by K.Tsuru  */
/*
This program compares SComplex class with C++ complex one.
It has been tested on gcc.
*/
#include "sn.h"
static const char* const title = "A test program for SComplex class";
void ShowTitle(const char* const fn){
	string msg("****************** test of ");
	msg += string(fn);
	msg += " ********************";
	cout << msg << endl;
}
void deMoivreFormula(){
	cerr << "It confirms de Moivre's fromula." << endl;
	SDouble x = Pi()/16.0;
	SComplex zpow, zn, z, iz, dif;
	z.Set(Cos(x), Sin(x));

	int n, N = 10;
	x.SetFormat(10,11);
	cout << "  n   difference between (cos x + i* sin x)^n and cos n*x + i*sin n*x ";
	cout << "by x= " << x << endl;
	for(n = -N; n<= N; n++){
		zpow = Cpow(z, n);
		zn = Cpolar(1.0, n*x);
		dif = zpow - zn;
		cout << setw(5) << n << "  " << dif << endl;
	}
}
void SomeIdentities(){
	cerr << "It confirms some identities." << endl;
	SComplex z(Pi(), Sqrt(2.0)), iz, dif;
	iz = IU*z;// IU = sqrt(-1.0)

	cerr << "sinh(i*z)-i*sin(z) = 0" << endl;
	dif = Csinh(iz) - IU*Csin(z);
	cout << dif << endl;

	cerr << "cosh(i*z)-cos(z) = 0" << endl;
	dif = Ccosh(iz) - Ccos(z);
	cout << dif << endl;

	cerr << "tanh(i*z)-i*tan(z) = 0" << endl;
	dif = Ctanh(iz) - IU*Ctan(z);
	cout << dif << endl;
	cerr << "Did you get values very close to zero ?" << endl;
}
void TestCtanh(){
	ShowTitle("Ctanh");
	SComplex z, q;
	SDouble theta;
	theta.SetFormat(0,8);
	int i, N = 10;
	double cf = -M_PI;
	complex <double> dz, dq, dif;
	cout << "theta |SDouble to double value |   double value     | difference between two" << endl;
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(theta, cf*theta);
		dz = complexD(z); //complex <double>(doubleD(theta), cf*doubleD(theta));
		dq = sinh(dz)/cosh(dz);
		q = Ctanh(z);
		dif = dq -complexD(q);
		cout << fixed << setprecision(1) << doubleD(theta) << " ";
		cout << setprecision(7) << complexD(q) << " " << dq << " ";
		cout << scientific << setprecision(5) << complexD(dif) << endl;
	}
}
void TestCtan(){
	ShowTitle("Ctan");
	SComplex z, q;
	SDouble theta;
	int i, N = 10;
	double cf = -M_PI;
	complex <double> dz, dq, dif;
	cout << "theta |SDouble to double value |   double value     | difference between two" << endl;
	for(i=-N; i<= N;i++){
		theta = 2*i/(double)N;
		z = SComplex(theta, cf*theta);
		dz =  complexD(z);// complex <double>(doubleD(theta), cf*doubleD(theta));
		dq = sin(dz)/cos(dz);
		q = Ctan(z);
		dif = dq -complexD(q);
		cout << fixed << setprecision(1) << doubleD(theta) << " ";
		cout << setprecision(7) << complexD(q) << " " << dq << " ";
		cout << scientific << setprecision(5) << complexD(dif) << endl;
	}
}

int main(){
	RealSize C;
	C.SetEffFig(30);

	ErrorPuts(title);

	TestCtan();			Wait();
	TestCtanh();		Wait();
	deMoivreFormula();	Wait();
	SomeIdentities();	Wait();
	C.SetEffFig(0);
	return 0;
}
