/*  spowtest.cpp by K.Tsuru  */
/*
A test program of
==================================================
template <class T> T PowUL(const T& x, ulong n) {
	T y(1), z(x);

	while(1){
		if( n & 1 ) y *= z;
		n /= 2;
		if( !n ) break;
		z *= z;
	}
	return y;
}
==================================================
in "snmath.h"
*/

#include "sn.h"
bool isSmall(const SDouble x) {
	static const SDouble err = 1.0e-11;
	return ( Dabs(x) < err );
}
// std::swap(a,b) can be used;
void swapTest(){
	SDouble a = Sqrt(2.0), b = Sqrt(3.0);
	cout << " Before swap(a, b)" << endl;
	cout << a << endl << b << endl;
	swap(a, b);
	cout << " After swap(a, b)" << endl;
	cout << a << endl << b << endl;
}
void compareWithDouble() {
	int N=50, i = N;
	SDouble rs,R1,R2,diff;
	cout.precision(7);
	double rd,r1,r2;
	while(i--){
		R1 = Drand(); R2 = Drand();
		r1 = doubleD(R1); r2 = doubleD(R2);
		rs = Pow(R1,R2); rd = pow(r1,r2);
  		diff = (rd != 0.0) ? (1.0 - rs/rd) : 0.0;
		cout << diff << endl;
		if(!isSmall(diff)){
			cerr << " diff = " << diff <<endl;
			cerr << " error is large" << endl; Wait();
		}
	}
}

int main() {
	int j;
	cout << "dpow10(n) test" << endl;
	for(j = -10; j<=10; j++) cout << dpow10(j) << endl;
	cout << "ipow10(n) test" << endl;
	for(j = -10; j<10; j++) cout << ipow10(j) << endl;
	cout << "ceilpow2(n) test" << endl;
	for(j = 0; j<15; j++) cout << ceilpow2(j) << endl;
	
	Wait();

	compareWithDouble();
	swapTest();
	Wait();

	double d;
	d = PowL(2.0, 100);
	cout << "2^100 = " << d << endl;
	SLong a(2);
	cout << a << " ^1000 = " << PowL(a,1000) << endl;
	cout << a << " ^1000 = " << Pow2(1000) << endl;

	SFraction b(2, 5), r;
	int p = -10;
	r = PowL(b, p);
	cout << b << "^(" << p << ")=" << r << endl; // This is Ok
//	cout << Pow(b,10) << endl; // direct output causes an error

	SDouble sq2 = Sqrt(2.0), x, y, e = E(), pi = Pi();
	y = Pow(pi, sq2);
	cout <<" pi^sqrt(2) = "<< y << endl;
	y = Pow(1/sq2, 2);
	cout << " y= "<< y << " = 0.5?" << endl;
	cout << "y = exp(sqrt(2)) and x = Pow(e, sqrt(2) )" << endl;
	y = Exp(sq2);
	x = Pow(e, sq2);
	cout << "x-y = " << x-y << " = 0?"<< endl;
	Ending();
}
